<?php

declare(strict_types=1);

namespace Drupal\webex_client\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\webex_client\AuthServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Returns responses for webex routes.
 *
 * This controller handles the OAuth callback process for Webex, including
 * error handling and successful login responses. It is responsible for
 * redirecting users based on the outcome of the OAuth authentication process.
 */
final class OAuthCallbackController extends ControllerBase {

  /**
   * Constructs a new OAuthCallbackController object.
   *
   * @param \Drupal\webex_client\AuthServiceInterface $webexAuth
   *   The authentication service for Webex.
   */
  public function __construct(
    private readonly AuthServiceInterface $webexAuth,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('webex_client.auth'),
    );
  }

  /**
   * Builds the response for the OAuth callback.
   *
   * This method processes the incoming request, checks for errors, and handles
   * the OAuth token exchange. It redirects the user to the appropriate route
   * based on the authentication outcome.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The HTTP request object containing the query parameters.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response to the appropriate URL.
   */
  public function __invoke(Request $request): RedirectResponse {
    if ($request->query->has('error')) {
      $callback_url = Url::fromRoute(
        'entity.webex.add_form',
        [],
        ['query' => $request->query->all()]
      )->toString();
      return new RedirectResponse($callback_url);
    }

    $code = $request->query->get('code');
    $state = $request->query->get('state');

    if (!$this->webexAuth->accessToken($code, $state)) {
      $this->messenger()->addError(
        $this->t('Failed to log in with the Webex platform.')
      );
    }
    else {
      $this->messenger()->addMessage(
        $this->t('Logged in successfully with the Webex platform.')
      );
    }

    $callback_url = Url::fromRoute('entity.webex.collection')->setAbsolute()->toString();
    return new RedirectResponse($callback_url);
  }

}
